@@ -31,6 +31,7 @@
 #include "month_view.h"
 
 #include <libdisplaymigration/displaymigration.h>
+#include <gpe/pim-categories.h>
 
 #include <locale.h>
 
@@ -197,6 +198,9 @@ main (int argc, char *argv[])
   if (event_db_start () == FALSE)
     exit (1);
 
+  if (gpe_pim_categories_init () == FALSE)
+    exit (1);
+
   while ((option_letter = getopt (argc, argv, "s:e:")) != -1)
     {
       if (option_letter == 's')
